@@ -32,15 +32,14 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DHT_ROUTING_TABLE_SERIALIZER_H_
-#define _D_DHT_ROUTING_TABLE_SERIALIZER_H_
+#ifndef D_DHT_ROUTING_TABLE_SERIALIZER_H
+#define D_DHT_ROUTING_TABLE_SERIALIZER_H
 
 #include "common.h"
 
 #include <vector>
-#include <iosfwd>
-
-#include "SharedHandle.h"
+#include <string>
+#include <memory>
 
 namespace aria2 {
 
@@ -50,21 +49,21 @@ class DHTRoutingTableSerializer {
 private:
   int family_;
 
-  SharedHandle<DHTNode> localNode_;
+  std::shared_ptr<DHTNode> localNode_;
 
-  std::vector<SharedHandle<DHTNode> > nodes_;
+  std::vector<std::shared_ptr<DHTNode> > nodes_;
 public:
   DHTRoutingTableSerializer(int family);
 
   ~DHTRoutingTableSerializer();
 
-  void setLocalNode(const SharedHandle<DHTNode>& localNode);
+  void setLocalNode(const std::shared_ptr<DHTNode>& localNode);
 
-  void setNodes(const std::vector<SharedHandle<DHTNode> >& nodes);
+  void setNodes(const std::vector<std::shared_ptr<DHTNode> >& nodes);
 
-  void serialize(std::ostream& o);
+  void serialize(const std::string& filename);
 };
 
 } // namespace aria2
 
-#endif // _D_DHT_ROUTING_TABLE_SERIALIZER_H_
+#endif // D_DHT_ROUTING_TABLE_SERIALIZER_H
