@@ -41,71 +41,71 @@
 
 namespace aria2 {
 
-#define CHANGE_INTERVAL_SEC 15
+#define WINDOW_MSEC 15000
 
-SpeedCalc::SpeedCalc():sw(0), maxSpeed(0), prevSpeed(0), accumulatedLength(0),
-                       nextInterval(CHANGE_INTERVAL_SEC)
+SpeedCalc::SpeedCalc()
+  : accumulatedLength_(0),
+    bytesWindow_(0),
+    maxSpeed_(0)
+{}
+
+void SpeedCalc::reset()
 {
-  std::fill(&lengthArray[0], &lengthArray[2], 0);
+  timeSlots_.clear();
+  start_ = global::wallclock();
+  accumulatedLength_ = 0;
+  bytesWindow_ = 0;
+  maxSpeed_ = 0;
 }
 
-void SpeedCalc::reset() {
-  std::fill(&lengthArray[0], &lengthArray[2], 0);
-  std::fill(&cpArray[0], &cpArray[2], global::wallclock);
-  sw = 0;
-  maxSpeed = 0;
-  prevSpeed = 0;
-  start = global::wallclock;
-  accumulatedLength = 0;
-  nextInterval = CHANGE_INTERVAL_SEC;
-}
-
-unsigned int SpeedCalc::calculateSpeed() {
-  int64_t milliElapsed = cpArray[sw].differenceInMillis(global::wallclock);
-  if(milliElapsed) {
-    unsigned int speed = lengthArray[sw]*1000/milliElapsed;
-    prevSpeed = speed;
-    maxSpeed = std::max(speed, maxSpeed);
-    if(isIntervalOver(milliElapsed)) {
-      changeSw();
+void SpeedCalc::removeStaleTimeSlot(int64_t now)
+{
+  while(!timeSlots_.empty()) {
+    if(now - timeSlots_[0].first <= WINDOW_MSEC) {
+      break;
+    } else {
+      bytesWindow_ -= timeSlots_[0].second;
+      timeSlots_.pop_front();
     }
-    return speed;
-  } else {
-    return prevSpeed;
   }
 }
 
-void SpeedCalc::update(size_t bytes) {
-  accumulatedLength += bytes;
-  std::transform(&lengthArray[0], &lengthArray[2], &lengthArray[0],
-                 std::bind1st(std::plus<uint64_t>(), (uint64_t)bytes));
-  if(isIntervalOver()) {
-    changeSw();
-  }
-}
-
-bool SpeedCalc::isIntervalOver() const {
-  return nextInterval <= cpArray[sw].difference(global::wallclock);
-}
-
-bool SpeedCalc::isIntervalOver(int64_t milliElapsed) const
+int SpeedCalc::calculateSpeed()
 {
-  return nextInterval <= milliElapsed/1000;
+  int64_t now = global::wallclock().getTimeInMillis();
+  removeStaleTimeSlot(now);
+  if(timeSlots_.empty()) {
+    return 0;
+  }
+  int64_t elapsed = now - timeSlots_[0].first;
+  if(elapsed <= 0) {
+    elapsed = 1;
+  }
+  int speed = bytesWindow_*1000/elapsed;
+  maxSpeed_ = std::max(speed, maxSpeed_);
+  return speed;
 }
 
-void SpeedCalc::changeSw() {
-  lengthArray[sw] = 0;
-  cpArray[sw] = global::wallclock;
-  sw ^= 0x01;
-  nextInterval = cpArray[sw].difference(global::wallclock)+CHANGE_INTERVAL_SEC;
+void SpeedCalc::update(size_t bytes)
+{
+  int64_t now = global::wallclock().getTimeInMillis();
+  removeStaleTimeSlot(now);
+  if(timeSlots_.empty() || now/1000 != timeSlots_.back().first/1000) {
+    timeSlots_.push_back(std::make_pair(now, bytes));
+  } else {
+    timeSlots_.back().second += bytes;
+  }
+  bytesWindow_ += bytes;
+  accumulatedLength_ += bytes;
 }
 
-unsigned int SpeedCalc::calculateAvgSpeed() const {
-  uint64_t milliElapsed = start.differenceInMillis(global::wallclock);
-
-  // if milliElapsed is too small, the average speed is rubish, better return 0
+int SpeedCalc::calculateAvgSpeed() const
+{
+  int64_t milliElapsed = start_.differenceInMillis(global::wallclock());
+  // if milliElapsed is too small, the average speed is rubish, better
+  // return 0
   if(milliElapsed > 4) {
-    unsigned int speed = accumulatedLength*1000/milliElapsed;
+    int speed = accumulatedLength_*1000/milliElapsed;
     return speed;
   } else {
     return 0;
